/* include/barcelona/gotype.h
 *
 * Declarations for different TomTom GO types.
 *
 * Copyright (C) 2004,2005,2006,2007,2008,2009,2010 TomTom BV <http://www.tomtom.com/>
 * Authors: Dimitry Andric <dimitry.andric@tomtom.com>
 *          Jeroen Taverne <jeroen.taverne@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __INCLUDE_BARCELONA_GOBOARD_H
#define __INCLUDE_BARCELONA_GOBOARD_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* GO id pin bits */
#define GOBOARD_GPIO			(1 << 23)              
#define GOBOARD_ID0_LO			0
#define GOBOARD_ID0_HI			(1 << 0)
#define GOBOARD_ID1_LO			0
#define GOBOARD_ID1_HI			(1 << 1)
#define GOBOARD_ID2_LO			0
#define GOBOARD_ID2_HI			(1 << 2)
#define GOBOARD_ID3_LO			0 
#define GOBOARD_ID3_HI			(1 << 3) 
#define GOBOARD_ID4_LO			0 
#define GOBOARD_ID4_HI			(1 << 4) 

/* GO screen type bits */
#define GOBOARD_NEC_TFT			0
#define GOBOARD_SAMSUNG_TFT		(1 << 5)

/* GO CPU type bits */
#define GOBOARD_S3C2410			0
#define GOBOARD_S3C2440			(1 << 6)
#define GOBOARD_S3C2442			(1 << 7)
#define GOBOARD_S3C2412			((1 << 6) | (1 << 7))
#define GOBOARD_S3C2443			(1 << 8)

/* Devices  having hardware ID in GPIO pins */
#define GOBOARD_CLASSIC		(GOBOARD_GPIO |                  GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2410 | GOBOARD_NEC_TFT)
#define GOBOARD_M100		(GOBOARD_GPIO |                  GOBOARD_ID1_LO | GOBOARD_ID0_HI | GOBOARD_S3C2410 | GOBOARD_NEC_TFT)

#define GOBOARD_M300		(GOBOARD_GPIO |                  GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2440 | GOBOARD_NEC_TFT)
#define GOBOARD_M500		(GOBOARD_GPIO |                  GOBOARD_ID1_HI | GOBOARD_ID0_LO | GOBOARD_S3C2440 | GOBOARD_NEC_TFT)
#define GOBOARD_ATLANTA		(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_LO | GOBOARD_ID0_HI | GOBOARD_S3C2440 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_ABERDEEN	(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2440 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_NEWCASTLE	(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_HI | GOBOARD_ID0_LO | GOBOARD_S3C2440 | GOBOARD_SAMSUNG_TFT)

#define GOBOARD_VALENCIA	(GOBOARD_GPIO |                  GOBOARD_ID1_LO | GOBOARD_ID0_LO | GOBOARD_S3C2442 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_MURCIA		(GOBOARD_GPIO |                  GOBOARD_ID1_LO | GOBOARD_ID0_HI | GOBOARD_S3C2442 | GOBOARD_SAMSUNG_TFT)

#define GOBOARD_EDINBURGH	(GOBOARD_GPIO | GOBOARD_ID2_LO | GOBOARD_ID1_LO | GOBOARD_ID0_LO | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_COLOGNE		(GOBOARD_GPIO | GOBOARD_ID2_LO | GOBOARD_ID1_LO | GOBOARD_ID0_HI | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_CORK		(GOBOARD_GPIO | GOBOARD_ID2_LO | GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_LIMERICK	(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_LO | GOBOARD_ID0_LO | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_CASABLANCA_GL	(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_LO | GOBOARD_ID0_HI | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_CASABLANCA_SIRF	(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_HI | GOBOARD_ID0_LO | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_CORKII		(GOBOARD_GPIO | GOBOARD_ID2_HI | GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)
#define GOBOARD_KNOCK		(GOBOARD_GPIO | GOBOARD_ID2_LO | GOBOARD_ID1_HI | GOBOARD_ID0_LO | GOBOARD_S3C2412 | GOBOARD_SAMSUNG_TFT)

#define GOBOARD_MILAN		(GOBOARD_GPIO |                  GOBOARD_ID1_HI | GOBOARD_ID0_HI | GOBOARD_S3C2443 | GOBOARD_SAMSUNG_TFT)

/* Devices having hardware ID in NOR FLASH */
#define GOBOARD_PALERMO		1
#define GOBOARD_ROME		2
#define GOBOARD_DUBLIN		3
#define GOBOARD_CAGLIARI	4
#define GOBOARD_CASABLANCA_CN	5
#define GOBOARD_CAIRNS		6
#define GOBOARD_MARIGOT		7
#define GOBOARD_PALERMOC	8
#define GOBOARD_ROMEC		9
#define GOBOARD_MODENA		10
//#define GOBOARD_ATLAS3_EVB	11
//#define GOBOARD_REDWOOD	12
//#define GOBOARD_REDWOOD2	13
#define GOBOARD_FAYETTEVILLE_CASABLANCA		14
#define GOBOARD_FAYETTEVILLE_LIMERICK		15
#define GOBOARD_FLORENCE	16
#define GOBOARD_PAOLA		17
#define GOBOARD_LIVORNO		18
#define GOBOARD_DURBAN		19
#define GOBOARD_TREVISO		20
#define GOBOARD_XIAMEN		21
#define GOBOARD_PUDONG		22
#define GOBOARD_NANCHANG	23
#define GOBOARD_TSINGTAO	24
#define GOBOARD_AUSTIN		25
#define GOBOARD_BERGAMO		28
#define GOBOARD_RIDER3		29
#define GOBOARD_BOSTON		30
#define GOBOARD_ACTON		31
#define GOBOARD_ANQING		32

/* Devices having hardware SUB_ID in NOR FLASH */
#define GOBOARD_FLASH_SUB_ID_1        1

/* Marigot board identifiers */
#define GOBOARD_MARIGOT_M1       (GOBOARD_MARIGOT | ( GOBOARD_FLASH_SUB_ID_1 << 16))
/* Cairns board identifiers */
#define GOBOARD_CAIRNS_C1	(GOBOARD_CAIRNS | ( GOBOARD_FLASH_SUB_ID_1 << 16))

/* Atlanta board identifiers */
#define GOBOARD_BILBAO		(GOBOARD_ATLANTA | GOBOARD_ID4_HI | GOBOARD_ID3_HI)
#define GOBOARD_GLASGOW		(GOBOARD_ATLANTA | GOBOARD_ID4_LO | GOBOARD_ID3_HI)
#define GOBOARD_LISBON		(GOBOARD_ATLANTA | GOBOARD_ID4_HI | GOBOARD_ID3_LO)

/* Cork  board identifiers	*/
#define GOBOARD_CORK_Z1Z2	(GOBOARD_CORK | GOBOARD_ID3_LO )	/* With 1GB/512 MB MoviNAND */
#define GOBOARD_CORK_Z3		(GOBOARD_CORK | GOBOARD_ID3_HI )	/* WithOUT MoviNAND */

/* Limerick board identifiers */
#define GOBOARD_LIMERICK_L1L2	(GOBOARD_LIMERICK | GOBOARD_ID3_LO | GOBOARD_ID4_LO )	/* With 1GB / 512 MB MoviNAND */
#define GOBOARD_LIMERICK_L3	(GOBOARD_LIMERICK | GOBOARD_ID3_HI | GOBOARD_ID4_LO )	/* WithOUT MoviNAND */
#define GOBOARD_LIMERICK_L4	(GOBOARD_LIMERICK | GOBOARD_ID3_LO | GOBOARD_ID4_HI )	/* Like L1 but with 64 MB SDRAM */
#define GOBOARD_LIMERICK_L5	(GOBOARD_LIMERICK | GOBOARD_ID3_HI | GOBOARD_ID4_HI )	/* Like L3 but with 64 MB SDRAM */

/* Milan board identifiers */
#define GOBOARD_MILAN_M2	(GOBOARD_MILAN | GOBOARD_ID4_LO | GOBOARD_ID3_LO)
#define GOBOARD_MILAN_M4	(GOBOARD_MILAN | GOBOARD_ID4_LO | GOBOARD_ID3_HI)
#define GOBOARD_MILAN_M6	(GOBOARD_MILAN | GOBOARD_ID4_HI | GOBOARD_ID3_LO)
#define GOBOARD_MILAN_M8	(GOBOARD_MILAN | GOBOARD_ID4_HI | GOBOARD_ID3_HI)

/* Modena softboard identifiers */
#define GOBOARD_MODENA_J1	(GOBOARD_MODENA | (1 << 16))
#define GOBOARD_MODENA_J2	(GOBOARD_MODENA | (2 << 16))
#define GOBOARD_MODENA_J3	(GOBOARD_MODENA | (3 << 16))
#define GOBOARD_MODENA_J4	(GOBOARD_MODENA | (4 << 16))
#define GOBOARD_MODENA_J5	(GOBOARD_MODENA | (5 << 16))
#define GOBOARD_MODENA_J6	(GOBOARD_MODENA | (6 << 16))
#define GOBOARD_MODENA_JA	(GOBOARD_MODENA | (0xa << 16))
#define GOBOARD_MODENA_JB	(GOBOARD_MODENA | (0xb << 16))
#define GOBOARD_MODENA_JC	(GOBOARD_MODENA | (0xc << 16))
/* with Modena with Baracuda */
#define GOBOARD_MODENA_J81	(GOBOARD_MODENA | (0x81 << 16))
#define GOBOARD_MODENA_J82	(GOBOARD_MODENA | (0x82 << 16))
#define GOBOARD_MODENA_J83	(GOBOARD_MODENA | (0x83 << 16))
#define GOBOARD_MODENA_J84	(GOBOARD_MODENA | (0x84 << 16))
#define GOBOARD_MODENA_J85	(GOBOARD_MODENA | (0x85 << 16))
#define GOBOARD_MODENA_J86	(GOBOARD_MODENA | (0x86 << 16))
#define GOBOARD_MODENA_J8A	(GOBOARD_MODENA | (0x8a << 16))
#define GOBOARD_MODENA_J8B	(GOBOARD_MODENA | (0x8b << 16))
#define GOBOARD_MODENA_J8C	(GOBOARD_MODENA | (0x8c << 16))

/* Palermo softboard identifiers */
#define GOBOARD_PALERMO_P1	(GOBOARD_PALERMO | (1 << 16))

/* Rome softboard identifiers */
#define GOBOARD_ROME_R1		(GOBOARD_ROME | (1 << 16))

/* Cagliari softboard identifiers */
#define GOBOARD_CAGLIARI_940_NC		(GOBOARD_CAGLIARI | (1 << 16))
#define GOBOARD_CAGLIARI_740_NC		(GOBOARD_CAGLIARI | (2 << 16))
#define GOBOARD_CAGLIARI_540_NC		(GOBOARD_CAGLIARI | (3 << 16))
#define GOBOARD_CAGLIARI_940_C_WEU	(GOBOARD_CAGLIARI | (4 << 16))
#define GOBOARD_CAGLIARI_740_C_WEU	(GOBOARD_CAGLIARI | (5 << 16))
#define GOBOARD_CAGLIARI_540_C_WEU	(GOBOARD_CAGLIARI | (6 << 16))
#define GOBOARD_CAGLIARI_940_C_US	(GOBOARD_CAGLIARI | (7 << 16))
#define GOBOARD_CAGLIARI_740_C_US	(GOBOARD_CAGLIARI | (8 << 16))
#define GOBOARD_CAGLIARI_540_C_US	(GOBOARD_CAGLIARI | (9 << 16))

/* Treviso softboard identifiers */
#define GOBOARD_TREVISO_940_NC		(GOBOARD_TREVISO | (1 << 16))
#define GOBOARD_TREVISO_740_NC		(GOBOARD_TREVISO | (2 << 16))
#define GOBOARD_TREVISO_540_NC		(GOBOARD_TREVISO | (3 << 16))
#define GOBOARD_TREVISO_940_C_WEU	(GOBOARD_TREVISO | (4 << 16))
#define GOBOARD_TREVISO_740_C_WEU	(GOBOARD_TREVISO | (5 << 16))
#define GOBOARD_TREVISO_540_C_WEU	(GOBOARD_TREVISO | (6 << 16))
#define GOBOARD_TREVISO_940_C_US	(GOBOARD_TREVISO | (7 << 16))
#define GOBOARD_TREVISO_740_C_US	(GOBOARD_TREVISO | (8 << 16))
#define GOBOARD_TREVISO_540_C_US	(GOBOARD_TREVISO | (9 << 16))

#define GOBOARD_XIAMEN_US		(GOBOARD_XIAMEN | (2 << 16))

#define GOBOARD_BERGAMO_US		(GOBOARD_BERGAMO | (2 << 16))
#define GOBOARD_CHELSEA4		(GOBOARD_BERGAMO | (3 << 16))
#define GOBOARD_CHELSEA5		(GOBOARD_ANQING | (1 << 16))

#define GOBOARD_FLORENCE_FLORIN		(GOBOARD_FLORENCE | (1 << 16))

/* Casablanca board identifiers */
/* 
 * NOTE: Casablanca can NOT identify movinand size using gpio
 * according to schematic, GPB8 / TYPE_SUB_ID0 / GOBOARD_ID3_HI should always be '0'
 */
#define GOBOARD_CASABLANCA_SIRF_Y1Y3	(GOBOARD_CASABLANCA_SIRF | GOBOARD_ID3_LO)
#define GOBOARD_CASABLANCA_GL_Y2Y4	(GOBOARD_CASABLANCA_GL   | GOBOARD_ID3_LO)
#define GOBOARD_CASABLANCA_SIRF_UNKNOWN	(GOBOARD_CASABLANCA_SIRF | GOBOARD_ID3_HI)
#define GOBOARD_CASABLANCA_GL_UNKNOWN	(GOBOARD_CASABLANCA_GL   | GOBOARD_ID3_HI)

/* Knock board identifiers */
#define GOBOARD_KNOCK_MOVINAND		(GOBOARD_KNOCK | GOBOARD_ID3_LO)
#define GOBOARD_KNOCK_NO_MOVINAND	(GOBOARD_KNOCK | GOBOARD_ID3_HI)

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCLUDE_BARCELONA_GOBOARD_H */

/* EOF */
